
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: Bagnon, IsAddOnLoaded, next

local unpack = unpack

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Integration Bagnon")

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnInitialize()
	self.itemSlots = {}
end

function M:OnEnable()
	if not IsAddOnLoaded("Bagnon") then
		self:RegisterEvent("ADDON_LOADED")
		return
	end

	if not self:IsHooked(Bagnon.ItemSlot, "New") then
		self:RawHook(Bagnon.ItemSlot, "New", "ItemSlot_New")
	end

	self:RegisterFrames()
	self:EnableBorders()
end

function M:OnDisable()
	self:DisableBorders()
end

function M:RegisterFrames()
	A:RegisterFrame("BagnonInventoryFrame1"):CreateBorder(0, -2, 0, 0)
	A:RegisterFrame("BagnonBankFrame1"):CreateBorder(0, -2, 0, 0)
	A:RegisterFrame("BagnonGuildFrame1"):CreateBorder(0, -2, 0, 0)
	A:RegisterFrame("BagnonVaultFrame1"):CreateBorder(0, -2, 0, 0)
end

-- Enable SixUI's borders on all known item slots.
function M:EnableBorders()
	for itemSlot in next, self.itemSlots do
		self:StyleItemSlot(itemSlot)
	end
end

-- Disable SixUI's borders on all known item slots.
function M:DisableBorders()
	for itemSlot in next, self.itemSlots do
		self:UnstyleItemSlot(itemSlot)
	end
end

-- Apply SixUI's style to an item slot.
function M:StyleItemSlot(itemSlot)
	B:CreateBorder(itemSlot, C.BORDER_PADDING):SetBackground(false)

	if not self.itemSlots[itemSlot] then
		self.itemSlots[itemSlot] = true
	end

	if not self:IsHooked(itemSlot, "UpdateBorder") then
		self:SecureHook(itemSlot, "UpdateBorder")
	end

	itemSlot.icon:SetTexCoord(unpack(C.ZOOM_TEX_COORD))
	itemSlot.QuestBorder:SetTexCoord(unpack(C.ZOOM_TEX_COORD))

	self:UpdateBorder(itemSlot)
end

-- Remove SixUI's style from an item slot.
function M:UnstyleItemSlot(itemSlot)
	-- M.StyleItemSlot()
	itemSlot:GetBorder():Hide()
	itemSlot.icon:SetTexCoord(0, 1, 0, 1)
	itemSlot.QuestBorder:SetTexCoord(0, 1, 0, 1)

	-- M.UpdateBorder()
	itemSlot.IconBorder:Show()
	itemSlot.NormalTexture:Show()
	itemSlot:EnableDrawLayer("BORDER")
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------

function M:ADDON_LOADED(event, name)
	if name == "Bagnon" then
		self:UnregisterEvent("ADDON_LOADED")
		self:OnEnable()
	end
end

-------------------------------------------------------------------------------
-- Hooks
-------------------------------------------------------------------------------

-- Use a hook to find newly created item slots.
function M:ItemSlot_New(...)
	local itemSlot = self.hooks[Bagnon.ItemSlot].New(...)
	self:StyleItemSlot(itemSlot)
	return itemSlot
end

-- Use a hook to synchronise SixUI's border color with Bagnon.
function M:UpdateBorder(itemSlot)
	local info = itemSlot:GetInfo()

	itemSlot.IconBorder:Hide()
	itemSlot.NormalTexture:Hide()

	if not info.icon then
		itemSlot:DisableDrawLayer("BORDER")
		itemSlot:GetBorder():SetColor()
	else
		local r, g, b = itemSlot.IconBorder:GetVertexColor()
		itemSlot:EnableDrawLayer("BORDER")
		itemSlot:GetBorder():SetColor(r, g, b)
	end
end
